/* AVRemote
   
 (c) 2011 Nederlands Instituut voor Mediakunst (NIMk)
     2011 Denis Roio <jaromil@nimk.nl>

     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
     published by the Free Software Foundation, either version 3 of the
     License, or (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU Affero General Public License for more details.
     
     You should have received a copy of the GNU Affero General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>

*/

#include <stdio.h>
#include <string.h>


#define MAX 256 // max XML string size

// returns a pointer to the current position in haystack
// for subsequent calls
char *extract_xml(char *dest, char *haystack, const char *needle) {
  char *p, *pp;

  p = strstr(haystack, needle);
  if (!p) {
    fprintf(stderr, "Bad reply to \"%s\":\n%s\n",needle, haystack);      
    return NULL;
    /* TODO parse XML correctly and handle standard errors:
     *
     * errorCode errorDescription Description
     * --------	---------------- -----------
     * 401 		Invalid Action 	No action by that name at this service.
     * 402 		Invalid Args 	Could be any of the following: not enough in args,
     * 							too many in args, no in arg by that name, 
     * 							one or more in args are of the wrong data type.
     * 403 		Out of Sync 	Out of synchronization.
     * 501 		Action Failed 	May be returned in current state of service
     * 							prevents invoking that action.
     * 600-699 	TBD 			Common action errors. Defined by UPnP Forum
     * 							Technical Committee.
     * 700-799 	TBD 			Action-specific errors for standard actions.
     * 							Defined by UPnP Forum working committee.
     * 800-899 	TBD 			Action-specific errors for non-standard actions. 
     * 							Defined by UPnP vendor.
     */
  }
  // as you can see, i had a lot of fun with pointers when i was a kid
  pp = p; do pp+=1; while (*pp != '>'); pp++;
  p = pp; do p+=1; while (*p != '<'); *p = 0;

  snprintf(dest,MAX-1,"%s",pp);

  return(p+1);

}

void GetCurrentTransportActions(char *res) {
  char actions[MAX];
  if( ! extract_xml(actions, res, "Actions") ) return;

  fprintf(stderr,"#\tActions\n");
  fprintf(stderr,"Act: %s\n", actions);
}

void GetDeviceCapabilities(char *res) {
  char play[MAX];
  char rec[MAX];
  char qual[MAX];
  char *p;

  p = extract_xml(play, res, "PlayMedia"); if(!p) return;
  p = extract_xml(rec,  p, "RecMedia");
  p = extract_xml(qual, p, "RecQualityModes");

  fprintf(stderr,"#\tPlay\tRec\tQuality\n");  
  fprintf(stderr,"Caps:\t%s\t%s\t%s\n",play,rec,qual);
}

void GetTransportInfo(char *res) {
  char state[MAX];
  char status[MAX];
  char speed[MAX];
  char *p;

  p = extract_xml(state, res, "CurrentTransportState"); if(!p) return;
  p = extract_xml(status, p, "CurrentTransportStatus");
  p = extract_xml(speed, p, "CurrentSpeed");

  fprintf(stderr,"#\tstate\tstatus\tspeed\n");
  fprintf(stderr,"TInfo:\t%s\t%s\t%s\n", state, status, speed);
}
